/*
 *  Shinobi Library
 *  Copyright (c) 1998 SEGA
 *
 *       Backup
 *
 *  Module  : Library Header File
 *  File    : sg_bup.h
 *  Create  : 1998-07-03
 *  Modify  : 1998-09-22
 *  Version : 1.04
 *  Note    :
 */

#ifndef _SG_BUP_H_
#define _SG_BUP_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * C VIRTUAL TYPES DEFINITIONS
 */

#include <sg_xpt.h>


/*
 *  TYPEDEFS
 */

/******** ^CX^v\ *********************************************/
typedef struct {
	Uint16 year;                /* N(1998`)                              */
	Uint8 month;                /* (1`12)                               */
	Uint8 day;                  /* (1`31)                               */
	Uint8 hour;                 /* (0`23)                               */
	Uint8 minute;               /* (0`59)                               */
	Uint8 second;               /* b(0`59)                               */
	Uint8 dayofweek;            /* j( = 0, = 1, ... y = 6)         */
} BUS_TIME;


/******** fBXN\ ***********************************************/
typedef struct {
	Uint8 volume[32];           /* {[f[^                        */
	Uint16 total_blocks;        /* SubN                            */
	Uint16 total_user_blocks;   /* S[U[ubN(őt@C)    */
	Uint16 free_blocks;         /* 󂫃ubN                          */
	Uint16 free_user_blocks;    /* 󂫃[U[ubN                  */
	Uint16 total_exe_blocks;    /* Sst@CpubN              */
	Uint16 free_exe_blocks;     /* 󂫎st@CpubN            */
	Uint16 block_size;          /* ubNTCY(512Œ)                 */
	Uint16 icon_no;             /* ACRԍ(0`255)                    */
	BUS_TIME time;              /* tH[}bgꂽ                  */
} BUS_DISKINFO;

/******** t@C\ ***********************************************/
typedef struct {
	Uint32 filesize;            /* t@CTCY(oCg)                */
	Uint16 blocks;              /* gpubN                          */
	Uint8 type;                 /* t@C^Cv(ʏ/s\)           */
	Uint8 copyflag;             /* Rs[tO(ff=Rs[s)             */
	Uint16 headerofs;           /* wb_ItZbg                        */
	BUS_TIME time;              /* ^CX^v                          */
} BUS_FILEINFO;



/******** obNAbvt@Cwb_ ***************************************/
typedef struct {
	char vms_comment[18];       /* VMSpRg                           */
	char btr_comment[34];       /* BOOT ROMpRg                      */
	Uint8 game_name[16];        /* Q[                                */
	void* icon_palette;         /* ACRpbgAhX                */
	void* icon_data;            /* ACRf[^AhX                  */
	Uint16 icon_num;            /* ACR                              */
	Uint16 icon_speed;          /* ACRAj[VXs[h          */
	void* visual_data;          /* rWAf[^AhX                */
	Uint16 visual_type;         /* rWA^Cv                        */
	Uint16 reserved;            /* \                                    */
	void* save_data;            /* AvP[VZ[uf[^AhX    */
	Uint32 save_size;           /* AvP[VZ[uf[^TCY      */
} BUS_BACKUPFILEHEADER;

/******** {[ACRt@C ***************************************/
typedef struct {
	char vms_comment[16];              /* VMSpRg                    */
	Uint8 mono_icon[128];              /* mNACRf[^           */
	Uint16 color_icon_palette[16];     /* J[ACRpbg           */
	Uint8 color_icon[512];             /* J[ACRf[^             */
	Uint8 reserved[1024-16-128-544-4]; /* \                             */
	Uint32 aligndata;                  /* 4oCgAĈ߂̃f[^    */
} BUS_VOLUMEICONFILE;


typedef void (*BU_INITCALLBACK)(void);
typedef Sint32 (*BU_COMPLETECALLBACK)(Sint32 drive, Sint32 op,
												Sint32 status, Uint32 param);
typedef Sint32 (*BU_PROGRESSCALLBACK)(Sint32 drive, Sint32 op,
												Sint32 count, Sint32 max);


/*
 *  PROTOTYPES
 */


/*------- ʏAvŎgp֐ ----------------------------------------*/
Sint32 buInit(Sint32 capacity, Sint32 drives, void* work,
													BU_INITCALLBACK func);
Sint32 buExit(void);
Sint32 buSaveFile(Sint32 drive, const char* fname, const void* buf,
									Uint32 nblock, BUS_TIME* time, Uint32 flag);
Sint32 buSaveExecFile(Sint32 drive, const char* fname, const void* buf,
									Uint32 nblock, BUS_TIME* time, Uint32 flag);
Sint32 buLoadFile(Sint32 drive, const char* fname, void* buf, Uint32 nblock);

Sint32 buDeleteFile(Sint32 drive, const char* fname);
Sint32 buDeleteFileEx(Sint32 drive, const char** fname, Sint32 nfiles);

void buSetCompleteCallback(BU_COMPLETECALLBACK func);
void buSetProgressCallback(BU_PROGRESSCALLBACK func);
Sint32 buIsReady(Sint32 drive);
Sint32 buIsFormat(Sint32 drive);
Sint32 buGetDiskFree(Sint32 drive, Sint32 type);
Sint32 buGetFileInfo(Sint32 drive, const char* fname, BUS_FILEINFO* finfo);
Sint32 buGetFileSize(Sint32 drive, const char* fname);
Sint32 buIsExistFile(Sint32 drive, const char* fname);
Sint32 buStat(Sint32 drive);
Sint32 buGetLastError(Sint32 drive);
Sint32 buMountDisk(Sint32 drive, void* addr, Sint32 size);
Sint32 buUnmountDisk(Sint32 drive);
Sint32 buRewriteExecFile(Sint32 drive, const char* fname, const void* buf,
												Uint32 start, Uint32 nblock);

/*------- Ƀt@C[eBeBŎgp֐ ----------------------*/
void buSetCallbackParam(Uint32 param);
Sint32 buMount(Sint32 drive);
Sint32 buRemount(Sint32 drive);
Sint32 buUnmount(Sint32 drive);
Sint32 buGetDiskInfo(Sint32 drive, BUS_DISKINFO* info);
Sint32 buFindFirstFile(Sint32 drive, char* fname);
Sint32 buFindNextFile(Sint32 drive, char* fname);
Sint32 buFindExecFile(Sint32 drive, char* fname);
Sint32 buSetFileAttr(Sint32 drive, const char* fname, Uint16 hdrofs,
															Uint8 copyflag);
Sint32 buFormatDisk(Sint32 drive, const Uint8* volume, Sint32 icon,
											const BUS_TIME* time, Uint32 flag);
Sint32 buRenameFile(Sint32 drive, const char* from, const char* to);
Sint32 buDefragDisk(Sint32 drive, void* work);
Sint32 buLoadFileEx(Sint32 drive, const char* fname, void* buf,
												Uint32 start, Uint32 nblock);

/*------- ̑̊֐ ----------------------------------------------------*/
void buExecServer(void);
Uint32 buGetVersion(void);
void buSetPrintFunc(void (*func)(const char* str));
void buSetRetryCount(Sint32 count);
Sint32 buGetRetryCount(void);
Uint16 buCalcCRC(const void* buf, Uint32 size);
Sint32 buMakeBackupFileImage(void* buf, const BUS_BACKUPFILEHEADER* hdr);
Sint32 buCalcBackupFileSize(Uint32 inum, Uint32 vtype, Uint32 size);
Sint32 buAnalyzeBackupFileImage(BUS_BACKUPFILEHEADER* hdr, const void* buf);


/******** [Ne *******************************************************/
#define BUD_WORK_GAP 32

#define BUM_WORK_SIZE(_cap_, _max_) \
	(((_cap_ * _max_) * 512) + (BUD_WORK_GAP * _max_))

/******** Z[ũtOw *********************************************/
#define BUD_FLAG_VERIFY       0x80000000    /* xt@Cw              */
#define BUD_FLAG_COPY(n)      (n & 0xff)    /* Rs[tO                */
#define BUD_FLAG_COPYDISABLE  0x000000ff    /* Rs[֎~tOt    */

/******** Ήőe *****************************************************/
#define BUD_CAPACITY_128KB   (1 + 1 +  13)  /* e128KB̂               */
#define BUD_CAPACITY_256KB   (1 + 2 +  30)  /* e256KB܂               */
#define BUD_CAPACITY_512KB   (1 + 4 +  60)  /* e512KB܂               */
#define BUD_CAPACITY_1MB     (1 + 8 + 113)  /* e1MB܂                 */

/******** ΉhCu *****************************************************/
#define BUD_USE_DRIVE_A1    (1 << 0)        /* |[gÅgfoCX1      */
#define BUD_USE_DRIVE_A2    (1 << 1)        /* |[gÅgfoCX2      */
#define BUD_USE_DRIVE_B1    (1 << 2)        /* |[gB̊gfoCX1      */
#define BUD_USE_DRIVE_B2    (1 << 3)        /* |[gB̊gfoCX2      */
#define BUD_USE_DRIVE_C1    (1 << 4)        /* |[gC̊gfoCX1      */
#define BUD_USE_DRIVE_C2    (1 << 5)        /* |[gC̊gfoCX2      */
#define BUD_USE_DRIVE_D1    (1 << 6)        /* |[gD̊gfoCX1      */
#define BUD_USE_DRIVE_D2    (1 << 7)        /* |[gD̊gfoCX2      */
#define BUD_USE_DRIVE_ALL   0xff            /* S                        */

/******** hCuԍ *****************************************************/
#define BUD_DRIVE_A1   0                    /* |[gÅgfoCX1      */
#define BUD_DRIVE_A2   1                    /* |[gÅgfoCX2      */
#define BUD_DRIVE_B1   2                    /* |[gB̊gfoCX1      */
#define BUD_DRIVE_B2   3                    /* |[gB̊gfoCX2      */
#define BUD_DRIVE_C1   4                    /* |[gC̊gfoCX1      */
#define BUD_DRIVE_C2   5                    /* |[gC̊gfoCX2      */
#define BUD_DRIVE_D1   6                    /* |[gD̊gfoCX1      */
#define BUD_DRIVE_D2   7                    /* |[gD̊gfoCX2      */

/******** t@C^Cv ***************************************************/
#define BUD_FILETYPE_NORMAL       0         /* ʏt@C                */
#define BUD_FILETYPE_EXECUTABLE   1         /* st@C                */


/******** Xe[^X *******************************************************/
#define BUD_STAT_READY       0x00000000     /* R}hs\            */
#define BUD_STAT_BUSY        0xffffffff     /* R}hs              */

/******** R[obNIy[Vp[^ *****************************/
#define BUD_OP_NOP             0x00000000   /* nop                         */
#define BUD_OP_MOUNT           0x00000001   /* buRemount()/ڑ            */
#define BUD_OP_UNMOUNT         0x00000002   /* ؒf                        */
#define BUD_OP_FORMATDISK      0x00000003   /* buFormatDisk()              */
#define BUD_OP_CLOSEFILE       0x00000004   /* buCloseFile()               */
#define BUD_OP_READFILE        0x00000005   /* buReadFile()                */
#define BUD_OP_WRITEFILE       0x00000006   /* buWriteFile()               */
#define BUD_OP_DELETEFILE      0x00000007   /* buDeleteFile()              */
#define BUD_OP_RENAMEFILE      0x00000008   /* buRenameFile()              */
#define BUD_OP_DEFRAGDISK      0x00000009   /* buDefragDisk()              */
#define BUD_OP_SETFILEATTR     0x0000000a   /* buSetFileAttr()             */
#define BUD_OP_LOADFILE        0x0000000b   /* buLoadFile()                */
#define BUD_OP_SAVEFILE        0x0000000c   /* buSaveFile()                */
#define BUD_OP_SAVEEXECFILE    0x0000000d   /* buSaveExecFile()            */
#define BUD_OP_LOADFILEEX      0x0000000e   /* buLoadFileEx()              */
#define BUD_OP_READIMAGE       0x0000000f   /* buReadDiskImage()           */
#define BUD_OP_WRITEIMAGE      0x00000010   /* buWriteDiskImage()          */
#define BUD_OP_DELETEFILEEX    0x00000011   /* buDeleteFileEx()            */
#define BUD_OP_CONNECT         0x00000012   /* ڑ(MOUNTO)               */
#define BUD_OP_REMOVE          0x00000013   /* ؒf                        */
#define BUD_OP_REWRITEEXECFILE 0x00000014   /* buRewriteExecFile()         */

/******** R[obNIy[Vp[^ *****************************/
#define BUD_CBRET_OK   0x00000000           /* p                    */

/******** ftOR[h ***********************************************/
#define BUD_DEFRAG_NOP       0x00000000     /* ftOł͂Ȃ          */
#define BUD_DEFRAG_READ      0x00000001     /* f[^ubN[h        */
#define BUD_DEFRAG_WRITE     0x00000002     /* f[^ubNCg        */
#define BUD_DEFRAG_DIR       0x00000003     /* fBNgGgXV    */
#define BUD_DEFRAG_FAT       0x00000004     /* FATXV                     */
#define BUD_DEFRAG_SYS       0x00000005     /* VXëXV            */

/******** G[R[h *****************************************************/
#define BUD_ERR_OK              0x00000000  /* G[Ȃ                  */
#define BUD_ERR_BUSY            0xffffffff  /* R}hs              */
#define BUD_ERR_INVALID_PARAM   0xffffff00  /* ֐p[^ُ        */
#define BUD_ERR_ILLEGAL_DISK    0xffffff81  /* ŏIubNs            */
#define BUD_ERR_UNKNOWN_DISK    0xffffff83  /* ΉĂȂfBXN      */
#define BUD_ERR_NO_DISK         0xffffff01  /* fBXNĂȂ      */
#define BUD_ERR_UNFORMAT        0xffffff03  /* tH[}bg              */
#define BUD_ERR_DISK_FULL       0xffffff04  /* fBXNt                */
#define BUD_ERR_FILE_NOT_FOUND  0xffffff05  /* t@CȂ      */
#define BUD_ERR_FILE_EXIST      0xffffff06  /* Õt@C    */
#define BUD_ERR_CANNOT_OPEN     0xffffff07  /* t@CJȂ          */
#define BUD_ERR_CANNOT_CREATE   0xffffff08  /* st@C쐬łȂ  */
#define BUD_ERR_EXECFILE_EXIST  0xffffff09  /* st@C݂      */
#define BUD_ERR_CANNOT_DELETE   0xffffff0a  /* t@CȂ      */
#define BUD_ERR_ACCESS_DENIED   0xffffff0b  /* t@CANZX        */
#define BUD_ERR_VERIFY          0xffffff10  /* xt@CG[            */
#define BUD_ERR_WRITE_ERROR     0xffffff40  /* CgG[                */
#define BUD_ERR_FILE_BROKEN     0xffffff41  /* t@CĂ        */
#define BUD_ERR_BUPFILE_CRC     0xffffff20  /* obNAbv`CRCG[   */
#define BUD_ERR_BUPFILE_ILLEGAL 0xffffff21  /* obNAbv`ł͂Ȃ    */
#define BUD_ERR_GENERIC         0xffff0000  /* `G[                */

/******** obNAbvt@C̃rWAf[^^Cv *********************/
#define BUD_VISUALTYPE_NONE   0             /* rWAf[^Ȃ        */
#define BUD_VISUALTYPE_A      1             /* ^CvA                     */
#define BUD_VISUALTYPE_B      2             /* ^CvB                     */
#define BUD_VISUALTYPE_C      3             /* ^CvC                     */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _SG_BUP_H_ */

