/*
 *-----------------------------------------------------------------------------
 *  FILE: backup.c
 *
 *      Copyright(c) 1998 SEGA System R&D Dept.
 *
 *  PURPOSE:
 *
 *      obNAbvCuTv(Im۔)
 *
 *  DESCRIPTION:
 *
 *  AUTHOR(S):
 *
 *      1997/9/21  S.Uchida Ver.0.00
 *
 *  MOD HISTORY:
 *
 *-----------------------------------------------------------------------------
 */

#include <shinobi.h>
#include "backup.h"

/*===============================================================*/
/* ΉobNAbvRAM̍őe                             */
/*===============================================================*/

#define MAX_CAPS BUD_CAPACITY_1MB


/*===============================================================*/
/* ΉobNAbvRAM̃hCu                           */
/*===============================================================*/

#define MAX_DRIVES 8
#define USE_DRIVES BUD_USE_DRIVE_ALL


/*===============================================================*/
/* obNAbvRAM̏Ԃi[Ă\                     */
/* (backup.h Q)                                             */
/*===============================================================*/

BACKUPINFO gBupInfo[8];



/*===============================================================*/
/*      X^eBbN֐̃vg^Cv錾                       */
/*===============================================================*/

static Sint32 BupComplete(Sint32 drive, Sint32 op, Sint32 stat, Uint32 param);
static Sint32 BupProgress(Sint32 drive, Sint32 op, Sint32 count, Sint32 max);
static void BupInitCallback(void);
static void ClearInfo(Sint32 drive);


/*===============================================================*/
/*      obNAbvCu                             */
/*      Imۂ邽߁A[NAhXɂNULLw   */
/*===============================================================*/

void BupInit(void)
{
	memset(gBupInfo, 0, sizeof(gBupInfo));
	buInit(MAX_CAPS, USE_DRIVES, NULL, BupInitCallback);
}


/*===============================================================*/
/*      obNAbvCuI                               */
/*===============================================================*/

void BupExit(void)
{
	do {} while (buExit() != BUD_ERR_OK);
}


/*===============================================================*/
/*      obNAbvRAM̏Ԃ擾                              */
/*===============================================================*/

const BACKUPINFO* BupGetInfo(Sint32 drive)
{
	return (const BACKUPINFO*)&gBupInfo[drive];
}


/*===============================================================*/
/*      t@C̃[h                                         */
/*===============================================================*/

Sint32 BupLoad(Sint32 drive, const char* fname, void* buf)
{
	return buLoadFile(drive, fname, buf, 0);
}


/*===============================================================*/
/*      t@C̃Z[u                                         */
/*===============================================================*/

enum {
	Sunday = 0,
	Monday,
	Tuesday,
	Wednesday,
	Thursday,
	Friday,
	Saturday
};

static const BUS_TIME gBupTime = {
	1998, 12, 31,	/* N */
	23, 59, 59,		/* b */
	Thursday,		/* j   */
};

Sint32 BupSave(Sint32 drive, const char* fname, void* buf, Sint32 nblock)
{
	return buSaveFile(drive, fname, buf, nblock, &gBupTime,
							BUD_FLAG_VERIFY | BUD_FLAG_COPY(0));
}


/*===============================================================*/
/*      t@C̍폜                                           */
/*===============================================================*/

Sint32 BupDelete(Sint32 drive, const char* fname)
{
	return buDeleteFile(drive, fname);
}

/*===============================================================*/
/*      [N̊mۂƃ}Eg                                   */
/*      ̏IA}EgR[obNԂĂ       */
/*      ܂łǂt@CANZX\ɂȂ܂             */
/*===============================================================*/

void BupMount(Sint32 drive)
{
	BACKUPINFO* info;

	info = &gBupInfo[drive];

	if (info->Work) return;		/* łɃ}Egς */

	info->Work = syMalloc(info->WorkSize);

	buMountDisk(drive, info->Work, info->WorkSize);
}

/*===============================================================*/
/*      A}Eg                                             */
/*      ȍ~̃hCuɂ̓ANZXł܂               */
/*      ĂуANZXꍇ́Ax[N̊mۂ         */
/*      }EgsĂ                                 */
/*      buUnmount()gp̒(dv)                          */
/*       EBUD_OP_UNMOUNTR[obN͔܂              */
/*       EKbuStat()ł̃hCuɃANZXsĂȂ  */
/*         ƂmFĂA}EgĂ            */
/*       EɃ[NJĂ肠܂                */
/*===============================================================*/

void BupUnmount(Sint32 drive)
{
	BACKUPINFO* info;
	Sint32 err;

	info = &gBupInfo[drive];

	if (info->Work == NULL) return;	/* }EgĂȂ */

	/* ̃hCuANZXłȂAA}Eg */
	/* [NJĂ͂ȂȂ                             */
	if (buStat(drive) == BUD_STAT_READY) {
		buUnmount(drive);
		syFree(info->Work);
		ClearInfo(drive);
	}
}

static void ClearInfo(Sint32 drive)
{
	BACKUPINFO* info;

	info = &gBupInfo[drive];
	info->ProgressCount = 0;
	info->ProgressMax = 0;
	info->Operation = 0;
	info->Ready = FALSE;
	info->IsFormat = FALSE;
	info->Work = NULL;
	memset(&info->DiskInfo, 0, sizeof(BUS_DISKINFO));
}

/*===============================================================*/
/*      G[R[h𕶎ɕϊ                               */
/*===============================================================*/

const char* BupGetErrorString(Sint32 err)
{
	switch (err) {
		case BUD_ERR_OK:				return "OK\0";
		case BUD_ERR_BUSY:				return "BUSY\0";
		case BUD_ERR_INVALID_PARAM:		return "INVALID PARAMETER\0";
		case BUD_ERR_ILLEGAL_DISK:		return "ILLEGAL DISK\0";
		case BUD_ERR_UNKNOWN_DISK:		return "UNKNOWN DISK\0";
		case BUD_ERR_NO_DISK:			return "NO DISK\0";
		case BUD_ERR_UNFORMAT:			return "UNFORMAT\0";
		case BUD_ERR_DISK_FULL:			return "DISK FULL\0";
		case BUD_ERR_FILE_NOT_FOUND:	return "FILE NOT FOUND\0";
		case BUD_ERR_FILE_EXIST:		return "FILE EXIST\0";
		case BUD_ERR_CANNOT_OPEN:		return "CANNOT OPEN\0";
		case BUD_ERR_CANNOT_CREATE:		return "CANNOT CREATE\0";
		case BUD_ERR_EXECFILE_EXIST:	return "EXECUTABLE FILE EXIST\0";
		case BUD_ERR_ACCESS_DENIED:		return "ACCESS DENIED\0";
		case BUD_ERR_VERIFY:			return "VERIFY ERROR\0";
		default:						return "GENERIC ERROR\0";
	}
}


/*===============================================================*/
/*      Iy[VR[h𕶎ɕϊ                       */
/*===============================================================*/

const char* BupGetOperationString(Sint32 op)
{
	switch (op) {
		case BUD_OP_CONNECT:		return "CONNECTED\0";
		case BUD_OP_MOUNT:			return "MOUNTED\0";
		case BUD_OP_UNMOUNT:		return "UNMOUNTED\0";
		case BUD_OP_FORMATDISK:		return "FORMATDISK\0";
		case BUD_OP_DELETEFILE:		return "DELETEFILE\0";
		case BUD_OP_LOADFILE:		return "LOADFILE\0";
		case BUD_OP_SAVEFILE:		return "SAVEFILE\0";
		default:					return "UNKNOWN OPERATION\0";
	}
}


/*===============================================================*/
/*      R[obN֐                                         */
/*===============================================================*/

/*
** IR[obN֐
*/

static void BupInitCallback(void)
{
	Sint32 i;

	/* Ao߃R[obN֐ݒ肷 */
	buSetCompleteCallback(BupComplete);
	buSetProgressCallback(BupProgress);
}


/*
** R[obN֐
*/

static Sint32 BupComplete(Sint32 drive, Sint32 op, Sint32 stat, Uint32 param)
{
	BACKUPINFO* info;
	Sint32 ret;


	info = &gBupInfo[drive];

	switch (op) {
		/* obNAbvRAMڑꂽ */
		case BUD_OP_CONNECT:
			info->Connect = TRUE;
			info->WorkSize = BUM_WORK_SIZE(stat, 1);
			info->Capacity = stat;
			break;
		/* obNAbvRAM}Egꂽ */
		case BUD_OP_MOUNT:
			if (stat == BUD_ERR_OK) {
				info->Ready = TRUE;
				ret = buGetDiskInfo(drive, &info->DiskInfo);
				if (ret == BUD_ERR_OK) {
					info->IsFormat = TRUE;
				}
				/* G[ԂNA */
				info->LastError = BUD_ERR_OK;
			}
			break;
		/* obNAbvRAMOꂽ */
		case BUD_OP_UNMOUNT:
			if (info->Work) syFree(info->Work);
			ClearInfo(drive);
			info->Connect = FALSE;
			break;
		default:
			/* G[Ԃi[ */
			info->LastError = stat;

			/* XV */
			buGetDiskInfo(drive, &info->DiskInfo);
	}

	/* Iy[VNA */
	info->Operation = 0;

	return BUD_CBRET_OK;
}


/*
** o߃R[obN֐
*/

static Sint32 BupProgress(Sint32 drive, Sint32 op, Sint32 count, Sint32 max)
{
	BACKUPINFO* info;

	info = &gBupInfo[drive];

	/* oߏԁAIy[VR[hi[ */
	info->ProgressCount = count;
	info->ProgressMax = max;
	info->Operation = op;

	return BUD_CBRET_OK;
}



/******************************* end of file *******************************/
